@@ -16,7 +16,7 @@
   GNU General Public License for more details.
 
 */
-/* $Id: command_reply.c,v 1.103 2002/05/03 16:36:49 priikone Exp $ */
+/* $Id: command_reply.c,v 1.115.2.2 2002/12/20 09:48:51 priikone Exp $ */
 /*
  * Command reply functions are "the otherside" of the command functions.
  * Reply to a command sent by server is handled by these functions.
@@ -241,13 +241,22 @@ silc_client_command_reply_whois_save(SilcClientCommandReplyContext cmd,
     client_entry->fingerprint_len = fingerprint_len;
   }
 
+  /* Take Requested Attributes if set. */
+  tmp = silc_argument_get_arg_type(cmd->args, 11, &len);
+  if (tmp) {
+    if (client_entry->attrs)
+      silc_attribute_payload_list_free(client_entry->attrs);
+    client_entry->attrs = silc_attribute_payload_parse(tmp, len);
+  }
+
   client_entry->status &= ~SILC_CLIENT_STATUS_RESOLVING;
 
   /* Notify application */
   if (!cmd->callbacks_count && notify)
-    COMMAND_REPLY((ARGS, client_entry, nickname, username, realname, 
+    COMMAND_REPLY((SILC_ARGS, client_entry, nickname, username, realname, 
 		   has_channels ? &channels : NULL, mode, idle, 
-		   fingerprint, has_user_modes ? &ch_user_modes : NULL));
+		   fingerprint, has_user_modes ? &ch_user_modes : NULL,
+		   client_entry->attrs));
 }
 
 /* Received reply for WHOIS command. This maybe called several times
@@ -339,7 +348,7 @@ SILC_CLIENT_CMD_REPLY_FUNC(whowas)
   /* Notify application. We don't save any history information to any
      cache. Just pass the data to the application for displaying on 
      the screen. */
-  COMMAND_REPLY((ARGS, client_entry, nickname, username, realname));
+  COMMAND_REPLY((SILC_ARGS, client_entry, nickname, username, realname));
 
   /* Pending callbacks are not executed if this was an list entry */
   if (cmd->status != SILC_STATUS_OK &&
@@ -413,7 +422,7 @@ silc_client_command_reply_identify_save(SilcClientCommandReplyContext cmd,
 
     /* Notify application */
     if (notify)
-      COMMAND_REPLY((ARGS, client_entry, name, info));
+      COMMAND_REPLY((SILC_ARGS, client_entry, name, info));
     break;
 
   case SILC_ID_SERVER:
@@ -432,11 +441,15 @@ silc_client_command_reply_identify_save(SilcClientCommandReplyContext cmd,
 	  COMMAND_REPLY_ERROR;
 	return;
       }
+    } else {
+      silc_client_update_server(client, conn, server_entry, name, info);
     }
 
+    server_entry->resolve_cmd_ident = 0;
+
     /* Notify application */
     if (notify)
-      COMMAND_REPLY((ARGS, server_entry, name, info));
+      COMMAND_REPLY((SILC_ARGS, server_entry, name, info));
     break;
 
   case SILC_ID_CHANNEL:
@@ -458,7 +471,7 @@ silc_client_command_reply_identify_save(SilcClientCommandReplyContext cmd,
 
     /* Notify application */
     if (notify)
-      COMMAND_REPLY((ARGS, channel_entry, name, info));
+      COMMAND_REPLY((SILC_ARGS, channel_entry, name, info));
     break;
   }
 
@@ -557,18 +570,19 @@ SILC_CLIENT_CMD_REPLY_FUNC(nick)
   /* Take the new nickname too */
   tmp = silc_argument_get_arg_type(cmd->args, 3, &len);
   if (tmp) {
-    silc_idcache_del_by_context(conn->client_cache, conn->local_entry);
+    silc_idcache_del_by_context(conn->internal->client_cache,
+				conn->local_entry);
     if (conn->nickname)
       silc_free(conn->nickname);
     conn->nickname = strdup(tmp);
     conn->local_entry->nickname = conn->nickname;
     silc_client_nickname_format(cmd->client, conn, conn->local_entry);
-    silc_idcache_add(conn->client_cache, strdup(tmp),
+    silc_idcache_add(conn->internal->client_cache, strdup(tmp),
                      conn->local_entry->id, conn->local_entry, 0, NULL);
   }
     
   /* Notify application */
-  COMMAND_REPLY((ARGS, conn->local_entry, conn->local_entry->nickname));
+  COMMAND_REPLY((SILC_ARGS, conn->local_entry, conn->local_entry->nickname));
 
  out:
   SILC_CLIENT_PENDING_EXEC(cmd, SILC_COMMAND_NICK);
@@ -626,7 +640,7 @@ SILC_CLIENT_CMD_REPLY_FUNC(list)
   }
 
   /* Notify application */
-  COMMAND_REPLY((ARGS, channel_entry, name, topic, usercount));
+  COMMAND_REPLY((SILC_ARGS, channel_entry, name, topic, usercount));
 
   /* Pending callbacks are not executed if this was an list entry */
   if (cmd->status != SILC_STATUS_OK &&
@@ -690,7 +704,7 @@ SILC_CLIENT_CMD_REPLY_FUNC(topic)
   }
   
   /* Notify application */
-  COMMAND_REPLY((ARGS, channel, topic));
+  COMMAND_REPLY((SILC_ARGS, channel, topic));
 
  out:
   SILC_CLIENT_PENDING_EXEC(cmd, SILC_COMMAND_TOPIC);
@@ -707,6 +721,7 @@ SILC_CLIENT_CMD_REPLY_FUNC(invite)
   SilcChannelID *channel_id;
   unsigned char *tmp;
   SilcUInt32 len;
+  SilcBufferStruct buf;
 
   if (cmd->error != SILC_STATUS_OK) {
     SAY(cmd->client, conn, SILC_CLIENT_MESSAGE_ERROR,
@@ -734,9 +749,11 @@ SILC_CLIENT_CMD_REPLY_FUNC(invite)
 
   /* Get the invite list */
   tmp = silc_argument_get_arg_type(cmd->args, 3, &len);
-
+  if (tmp)
+    silc_buffer_set(&buf, tmp, len);
+  
   /* Notify application */
-  COMMAND_REPLY((ARGS, channel, tmp));
+  COMMAND_REPLY((SILC_ARGS, channel, tmp ? &buf : NULL));
 
  out:
   SILC_CLIENT_PENDING_EXEC(cmd, SILC_COMMAND_INVITE);
@@ -758,7 +775,7 @@ SILC_CLIENT_CMD_REPLY_FUNC(kill)
   }
 
   /* Notify application */
-  COMMAND_REPLY((ARGS));
+  COMMAND_REPLY((SILC_ARGS));
 
  out:
   SILC_CLIENT_PENDING_EXEC(cmd, SILC_COMMAND_KILL);
@@ -818,7 +835,7 @@ SILC_CLIENT_CMD_REPLY_FUNC(info)
   }
 
   /* Notify application */
-  COMMAND_REPLY((ARGS, server, server->server_name, server->server_info));
+  COMMAND_REPLY((SILC_ARGS, server, server->server_name, server->server_info));
 
  out:
   SILC_CLIENT_PENDING_EXEC(cmd, SILC_COMMAND_INFO);
@@ -826,6 +843,38 @@ SILC_CLIENT_CMD_REPLY_FUNC(info)
   silc_client_command_reply_free(cmd);
 }
 
+/* Received reply to STATS command. */
+
+SILC_CLIENT_CMD_REPLY_FUNC(stats)
+{
+  SilcClientCommandReplyContext cmd = (SilcClientCommandReplyContext)context;
+  SilcClientConnection conn = (SilcClientConnection)cmd->sock->user_data;
+  unsigned char *tmp, *buf = NULL;
+  SilcUInt32 len, buf_len = 0;
+
+  if (cmd->error != SILC_STATUS_OK) {
+    SAY(cmd->client, conn, SILC_CLIENT_MESSAGE_ERROR,
+	"%s", silc_get_status_message(cmd->error));
+    COMMAND_REPLY_ERROR;
+    goto out;
+  }
+
+  /* Get server ID */
+  tmp = silc_argument_get_arg_type(cmd->args, 2, &len);
+  if (!tmp)
+    goto out;
+
+  /* Get statistics structure */
+  buf = silc_argument_get_arg_type(cmd->args, 3, &buf_len);
+
+  /* Notify application */
+  COMMAND_REPLY((SILC_ARGS, buf, buf_len));
+
+ out:
+  SILC_CLIENT_PENDING_EXEC(cmd, SILC_COMMAND_STATS);
+  silc_client_command_reply_free(cmd);
+}
+
 /* Received reply to PING command. The reply time is shown to user. */
 
 SILC_CLIENT_CMD_REPLY_FUNC(ping)
@@ -846,26 +895,26 @@ SILC_CLIENT_CMD_REPLY_FUNC(ping)
   curtime = time(NULL);
   id = silc_id_str2id(cmd->packet->src_id, cmd->packet->src_id_len,
 		      cmd->packet->src_id_type);
-  if (!id || !conn->ping) {
+  if (!id || !conn->internal->ping) {
     COMMAND_REPLY_ERROR;
     goto out;
   }
 
-  for (i = 0; i < conn->ping_count; i++) {
-    if (!conn->ping[i].dest_id)
+  for (i = 0; i < conn->internal->ping_count; i++) {
+    if (!conn->internal->ping[i].dest_id)
       continue;
-    if (SILC_ID_SERVER_COMPARE(conn->ping[i].dest_id, id)) {
-      diff = curtime - conn->ping[i].start_time;
+    if (SILC_ID_SERVER_COMPARE(conn->internal->ping[i].dest_id, id)) {
+      diff = curtime - conn->internal->ping[i].start_time;
       SAY(cmd->client, conn, SILC_CLIENT_MESSAGE_INFO, 
 	  "Ping reply from %s: %d second%s", 
-	  conn->ping[i].dest_name, diff, 
+	  conn->internal->ping[i].dest_name, diff, 
 	  diff == 1 ? "" : "s");
       
-      conn->ping[i].start_time = 0;
-      silc_free(conn->ping[i].dest_id);
-      conn->ping[i].dest_id = NULL;
-      silc_free(conn->ping[i].dest_name);
-      conn->ping[i].dest_name = NULL;
+      conn->internal->ping[i].start_time = 0;
+      silc_free(conn->internal->ping[i].dest_id);
+      conn->internal->ping[i].dest_id = NULL;
+      silc_free(conn->internal->ping[i].dest_name);
+      conn->internal->ping[i].dest_name = NULL;
       break;
     }
   }
@@ -873,7 +922,7 @@ SILC_CLIENT_CMD_REPLY_FUNC(ping)
   silc_free(id);
 
   /* Notify application */
-  COMMAND_REPLY((ARGS));
+  COMMAND_REPLY((SILC_ARGS));
 
  out:
   SILC_CLIENT_PENDING_EXEC(cmd, SILC_COMMAND_PING);
@@ -905,7 +954,7 @@ SILC_CLIENT_CMD_REPLY_FUNC(join)
   }
 
   argc = silc_argument_get_arg_num(cmd->args);
-  if (argc < 7 || argc > 14) {
+  if (argc < 7) {
     SAY(cmd->client, conn, SILC_CLIENT_MESSAGE_ERROR,
 	"Cannot join channel: Bad reply packet");
     COMMAND_REPLY_ERROR;
@@ -1050,7 +1099,7 @@ SILC_CLIENT_CMD_REPLY_FUNC(join)
     silc_client_save_channel_key(cmd->client, conn, keyp, channel);
 
   /* Notify application */
-  COMMAND_REPLY((ARGS, channel_name, channel, mode, 0, 
+  COMMAND_REPLY((SILC_ARGS, channel_name, channel, mode, 0, 
 		 keyp ? keyp->head : NULL, NULL,
 		 NULL, topic, hmac, list_count, client_id_list, 
 		 client_mode_list));
@@ -1101,7 +1150,7 @@ SILC_CLIENT_CMD_REPLY_FUNC(motd)
     while(cp[i] != 0) {
       if (cp[i++] == '\n') {
 	memset(line, 0, sizeof(line));
-	strncat(line, cp, i - 1);
+	silc_strncat(line, sizeof(line), cp, i - 1);
 	cp += i;
 	
 	if (i == 2)
@@ -1117,7 +1166,7 @@ SILC_CLIENT_CMD_REPLY_FUNC(motd)
   }
 
   /* Notify application */
-  COMMAND_REPLY((ARGS, motd));
+  COMMAND_REPLY((SILC_ARGS, motd));
 
  out:
   SILC_CLIENT_PENDING_EXEC(cmd, SILC_COMMAND_MOTD);
@@ -1150,7 +1199,7 @@ SILC_CLIENT_CMD_REPLY_FUNC(umode)
   conn->local_entry->mode = mode;
 
   /* Notify application */
-  COMMAND_REPLY((ARGS, mode));
+  COMMAND_REPLY((SILC_ARGS, mode));
 
  out:
   SILC_CLIENT_PENDING_EXEC(cmd, SILC_COMMAND_UMODE);
@@ -1191,7 +1240,7 @@ SILC_CLIENT_CMD_REPLY_FUNC(cmode)
     COMMAND_REPLY_ERROR;
     goto out;
   }
-  
+
   /* Get channel mode */
   tmp = silc_argument_get_arg_type(cmd->args, 3, NULL);
   if (!tmp) {
@@ -1205,7 +1254,7 @@ SILC_CLIENT_CMD_REPLY_FUNC(cmode)
   channel->mode = mode;
 
   /* Notify application */
-  COMMAND_REPLY((ARGS, channel, mode));
+  COMMAND_REPLY((SILC_ARGS, channel, mode));
 
   silc_free(channel_id);
 
@@ -1288,7 +1337,7 @@ SILC_CLIENT_CMD_REPLY_FUNC(cumode)
     chu->mode = mode;
 
   /* Notify application */
-  COMMAND_REPLY((ARGS, mode, channel, client_entry));
+  COMMAND_REPLY((SILC_ARGS, mode, channel, client_entry));
   silc_free(client_id);
   silc_free(channel_id);
   
@@ -1310,7 +1359,7 @@ SILC_CLIENT_CMD_REPLY_FUNC(kick)
   }
 
   /* Notify application */
-  COMMAND_REPLY((ARGS));
+  COMMAND_REPLY((SILC_ARGS));
 
  out:
   SILC_CLIENT_PENDING_EXEC(cmd, SILC_COMMAND_KICK);
@@ -1330,7 +1379,7 @@ SILC_CLIENT_CMD_REPLY_FUNC(silcoper)
   }
 
   /* Notify application */
-  COMMAND_REPLY((ARGS));
+  COMMAND_REPLY((SILC_ARGS));
 
  out:
   SILC_CLIENT_PENDING_EXEC(cmd, SILC_COMMAND_SILCOPER);
@@ -1350,7 +1399,7 @@ SILC_CLIENT_CMD_REPLY_FUNC(oper)
   }
 
   /* Notify application */
-  COMMAND_REPLY((ARGS));
+  COMMAND_REPLY((SILC_ARGS));
 
  out:
   SILC_CLIENT_PENDING_EXEC(cmd, SILC_COMMAND_OPER);
@@ -1371,7 +1420,7 @@ SILC_CLIENT_CMD_REPLY_FUNC(detach)
   }
 
   /* Notify application */
-  COMMAND_REPLY((ARGS));
+  COMMAND_REPLY((SILC_ARGS));
 
   /* Generate the detachment data and deliver it to the client in the
      detach client operation */
@@ -1400,7 +1449,7 @@ SILC_CLIENT_CMD_REPLY_FUNC(watch)
   }
 
   /* Notify application */
-  COMMAND_REPLY((ARGS));
+  COMMAND_REPLY((SILC_ARGS));
 
  out:
   SILC_CLIENT_PENDING_EXEC(cmd, SILC_COMMAND_WATCH);
@@ -1415,6 +1464,7 @@ SILC_CLIENT_CMD_REPLY_FUNC(ban)
   SilcChannelID *channel_id;
   unsigned char *tmp;
   SilcUInt32 len;
+  SilcBufferStruct buf;
 
   if (cmd->error != SILC_STATUS_OK) {
     SAY(cmd->client, conn, SILC_CLIENT_MESSAGE_ERROR,
@@ -1442,9 +1492,11 @@ SILC_CLIENT_CMD_REPLY_FUNC(ban)
   
   /* Get the ban list */
   tmp = silc_argument_get_arg_type(cmd->args, 3, &len);
-
+  if (tmp)
+    silc_buffer_set(&buf, tmp, len);
+  
   /* Notify application */
-  COMMAND_REPLY((ARGS, channel, tmp));
+  COMMAND_REPLY((SILC_ARGS, channel, tmp ? &buf : NULL));
 
  out:
   SILC_CLIENT_PENDING_EXEC(cmd, SILC_COMMAND_BAN);
@@ -1488,7 +1540,7 @@ SILC_CLIENT_CMD_REPLY_FUNC(leave)
   }
 
   /* Notify application */
-  COMMAND_REPLY((ARGS, channel));
+  COMMAND_REPLY((SILC_ARGS, channel));
 
  out:
   SILC_CLIENT_PENDING_EXEC(cmd, SILC_COMMAND_LEAVE);
@@ -1677,7 +1729,7 @@ silc_client_command_reply_users_save(SilcClientCommandReplyContext cmd,
 
   /* Notify application */
   if (notify)
-    COMMAND_REPLY((ARGS, channel, list_count, &client_id_list, 
+    COMMAND_REPLY((SILC_ARGS, channel, list_count, &client_id_list, 
 		   &client_mode_list));
 
  out:
@@ -1692,6 +1744,7 @@ SILC_CLIENT_CMD_REPLY_FUNC(users)
 {
   SilcClientCommandReplyContext cmd = (SilcClientCommandReplyContext)context;
   SilcClientConnection conn = (SilcClientConnection)cmd->sock->user_data;
+  SilcClientCommandReplyContext r = (SilcClientCommandReplyContext)context2;
 
   SILC_LOG_DEBUG(("Start"));
 
@@ -1702,6 +1755,13 @@ SILC_CLIENT_CMD_REPLY_FUNC(users)
     goto out;
   }
 
+  if (r && !silc_command_get_status(r->payload, NULL, &cmd->error)) {
+    SAY(cmd->client, conn, SILC_CLIENT_MESSAGE_ERROR,
+	"%s", silc_get_status_message(cmd->error));
+    COMMAND_REPLY_ERROR;
+    goto out;
+  }
+
   if (silc_client_command_reply_users_save(cmd, cmd->status, TRUE,
 					   silc_client_command_reply_users_cb,
 					   silc_client_command_reply_users))
@@ -1724,10 +1784,8 @@ SILC_CLIENT_CMD_REPLY_FUNC(getkey)
   SilcClientEntry client_entry;
   SilcServerID *server_id = NULL;
   SilcServerEntry server_entry;
-  SilcSKEPKType type;
-  unsigned char *tmp, *pk;
+  unsigned char *tmp;
   SilcUInt32 len;
-  SilcUInt16 pk_len;
   SilcIdType id_type;
   SilcPublicKey public_key = NULL;
 
@@ -1754,15 +1812,9 @@ SILC_CLIENT_CMD_REPLY_FUNC(getkey)
   /* Get the public key payload */
   tmp = silc_argument_get_arg_type(cmd->args, 3, &len);
   if (tmp) {
-    /* Decode the public key */
-    SILC_GET16_MSB(pk_len, tmp);
-    SILC_GET16_MSB(type, tmp + 2);
-    pk = tmp + 4;
-    
-    if (type == SILC_SKE_PK_TYPE_SILC)
-      if (!silc_pkcs_public_key_decode(pk, pk_len, &public_key))
-	public_key = NULL;
-  } 
+    if (!silc_pkcs_public_key_payload_decode(tmp, len, &public_key))
+      public_key = NULL;
+  }
    
   id_type = silc_id_payload_get_type(idp);
   if (id_type == SILC_ID_CLIENT) {
@@ -1774,8 +1826,16 @@ SILC_CLIENT_CMD_REPLY_FUNC(getkey)
       goto out;
     }
 
+    /* Save fingerprint */
+    if (!client_entry->fingerprint) {
+      client_entry->fingerprint = silc_calloc(20, sizeof(unsigned char));
+      client_entry->fingerprint_len = 20;
+      silc_hash_make(cmd->client->sha1hash, tmp + 4, len - 4,
+		     client_entry->fingerprint);
+    }
+
     /* Notify application */
-    COMMAND_REPLY((ARGS, id_type, client_entry, public_key));
+    COMMAND_REPLY((SILC_ARGS, id_type, client_entry, public_key));
   } else if (id_type == SILC_ID_SERVER) {
     /* Received server's public key */
     server_id = silc_id_payload_get_id(idp);
@@ -1786,7 +1846,7 @@ SILC_CLIENT_CMD_REPLY_FUNC(getkey)
     }
 
     /* Notify application */
-    COMMAND_REPLY((ARGS, id_type, server_entry, public_key));
+    COMMAND_REPLY((SILC_ARGS, id_type, server_entry, public_key));
   }
 
  out:
@@ -2019,7 +2079,7 @@ SILC_CLIENT_CMD_REPLY_FUNC(connect)
   }
 
   /* Notify application */
-  COMMAND_REPLY((ARGS));
+  COMMAND_REPLY((SILC_ARGS));
 
  out:
   SILC_CLIENT_PENDING_EXEC(cmd, SILC_COMMAND_PRIV_CONNECT);
@@ -2039,7 +2099,7 @@ SILC_CLIENT_CMD_REPLY_FUNC(close)
   }
 
   /* Notify application */
-  COMMAND_REPLY((ARGS));
+  COMMAND_REPLY((SILC_ARGS));
 
  out:
   SILC_CLIENT_PENDING_EXEC(cmd, SILC_COMMAND_PRIV_CLOSE);
@@ -2059,7 +2119,7 @@ SILC_CLIENT_CMD_REPLY_FUNC(shutdown)
   }
 
   /* Notify application */
-  COMMAND_REPLY((ARGS));
+  COMMAND_REPLY((SILC_ARGS));
 
  out:
   SILC_CLIENT_PENDING_EXEC(cmd, SILC_COMMAND_PRIV_SHUTDOWN);
